@@ -1,4 +1,4 @@
-#ident "@(#)do_stat.c	3.3 95/11/24 Copyright (c) Gert Doering"
+#ident "@(#)do_stat.c	4.1 97/01/12 Copyright (c) Gert Doering"
 
 /* do_stat.c
  *
@@ -28,7 +28,7 @@
 
 
 static boolean has_timeout;
-static RETSIGTYPE timeout()
+static RETSIGTYPE timeout(SIG_HDLR_ARGS)
 {
     has_timeout = TRUE;
 }
