@@ -1,4 +1,4 @@
-#ident "@(#)faxlib.c	3.6 96/05/26 Copyright (c) Gert Doering"
+#ident "@(#)faxlib.c	4.19 97/07/13 Copyright (c) Gert Doering"
 
 /* faxlib.c
  *
@@ -7,6 +7,7 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <unistd.h>
 #include <ctype.h>
 #include <string.h>
@@ -31,6 +32,8 @@ boolean fax_poll_req = FALSE;		/* poll requested */
 boolean	fhs_details = FALSE;		/* +FHS:xxx with detail info */
 int	fhs_lc, fhs_blc, fhs_cblc, fhs_lbc;
 
+int	modem_quirks = 0;		/* modem specials */
+
 
 /* wait for a given modem response string,
  * handle all the various class 2 / class 2.0 status responses
@@ -40,6 +43,9 @@ int	fhs_lc, fhs_blc, fhs_cblc, fhs_lbc;
 static void fwf_copy_remote_id _P1( (id), char * id )
 {
 int w = 0;
+
+    while ( isspace(*id) ) id++;	/* skip leading whitespace */
+
     while ( *id && w < sizeof(fax_remote_id)-1 )
     {
         if ( *id == '"' || *id == '\'' ) fax_remote_id[w++] = '_';
@@ -51,7 +57,7 @@ int w = 0;
 
 static boolean fwf_timeout = FALSE;
 
-static RETSIGTYPE fwf_sig_alarm()      	/* SIGALRM handler */
+static RETSIGTYPE fwf_sig_alarm(SIG_HDLR_ARGS)      	/* SIGALRM handler */
 {
     signal( SIGALRM, fwf_sig_alarm );
     lprintf( L_WARN, "Warning: got alarm signal!" );
@@ -204,8 +210,10 @@ int  ix;
 	if ( strcmp( line, "ERROR" ) == 0 ||
 	     strcmp( line, "NO CARRIER" ) == 0 ||
 	     strcmp( line, "BUSY" ) == 0 ||
-	     strcmp( line, "NO DIALTONE" ) == 0 )
+	     strcmp( line, "NO DIALTONE" ) == 0 ||
+	     strcmp( line, "NO DIAL TONE" ) == 0 )
 	{
+#if 0		/* not needed right now (fax_send_ppm), problem with USR!! */
 	    if ( modem_type == Mt_class2_0 )		/* valid response */
 	    {						/* in class 2.0! */
 		if ( strcmp( line, "ERROR" ) == 0 )
@@ -215,6 +223,7 @@ int  ix;
 		    return NOERROR;			/* !C2 */
 		}
 	    }
+#endif
 
 	    /* in class 2, one of the above codes means total failure */
 	    
@@ -222,7 +231,8 @@ int  ix;
 	    fax_hangup = 1;
 	    
 	    if ( strcmp(line, "BUSY") == 0 ) fax_hangup_code = FHUP_BUSY;
-	    else if (strcmp(line, "NO DIALTONE") == 0)
+	    else if (strcmp(line, "NO DIALTONE") == 0 ||
+		     strcmp(line, "NO DIAL TONE") == 0)
 	                                     fax_hangup_code = FHUP_NODIAL;
 	    else                             fax_hangup_code = FHUP_ERROR;
 	    
@@ -242,6 +252,19 @@ int  ix;
     return NOERROR;
 }
 
+/* (re-) initialize all global/static variables set by faxlib.c
+ * necessary if fax state machine runs multiple times, e.g. in vgetty
+ */
+void faxlib_init _P0( void )
+{
+    fax_hangup = 0;
+    fax_hangup_code = FHUP_UNKNOWN;
+    fax_page_tx_status = 0;
+    fax_to_poll = fax_poll_req = FALSE;
+    fhs_details = FALSE;
+    fax_remote_id[0] = '\0';
+}
+
 /* send a command string to the modem, terminated with the
  * MODEM_CMD_SUFFIX character / string from policy.h
  */
@@ -411,13 +434,39 @@ int fax_set_bor _P2( (fd, bor), int fd, int bor )
 
 Modem_type fax_get_modem_type _P2( (fd, mclass), int fd, char * mclass )
 {
+int rc;
+
     /* data modem? unknown mclass? handle as "auto" (for sendfax) */
     if ( strcmp( mclass, "cls2" ) != 0 &&
 	 strcmp( mclass, "c2.0" ) != 0 )
     {
 	mclass = "auto";
     }
-    
+
+    /* auto-identify */
+    if ( strcmp( mclass, "auto" ) == 0 &&
+	 mdm_identify( fd ) == NOERROR &&
+	 modem_type != Mt_unknown )
+    {
+	/* set up modem accordingly */
+	switch( modem_type )
+	{
+	    case Mt_class2_0: 
+		rc=mdm_command( "AT+FCLASS=2.0", fd ); 
+		break;
+	    case Mt_class2:
+		rc=mdm_command( "AT+FCLASS=2", fd ); 
+		break;
+	    default:
+		rc=NOERROR;
+		break;
+	}
+	if ( rc == NOERROR ) return modem_type;
+    }
+
+    /* not auto-identify, or initialization failed -> try "old way" 
+     */
+
     /* first of all, check for 2.0 */
     if ( strcmp( mclass, "auto" ) == 0 ||
 	 strcmp( mclass, "c2.0" ) == 0 )
@@ -440,3 +489,185 @@ Modem_type fax_get_modem_type _P2( (fd, mclass), int fd, char * mclass )
 
     return Mt_data;
 }			/* end fax_get_modem_type() */
+
+
+/* identify unknown modem via ATI code
+ *
+ * *very* preliminary - I'm experimenting...
+ */
+
+int mdm_identify _P1( (fd), int fd )
+{
+    char * l, *p;
+    char * mis = NULL;		/* more verbose modem ID string */
+
+    modem_type=Mt_unknown;
+
+    /* try ATI first, ATI<n> later to sub-divide results
+     */
+    l = mdm_get_idstring( "ATI", 1, fd );
+    
+    if ( l == NULL ) 
+    {
+	lprintf( L_WARN, "mdm_identify: can't get modem ID" );
+	return ERROR;
+    }
+
+    lprintf( L_NOISE, "mdm_identify: string '%s'", l );
+
+    /* all-numerical? */
+    p = l;
+    while( isdigit(*p) || isspace(*p) ) p++;
+
+    if ( *p == '\0' )		/* all-numeric */
+    {
+	int mid = atoi(l);	/* numerical modem ID... */
+
+	switch(mid)
+	{
+	  case 0:		/* empty string */
+	    lprintf( L_MESG, "got no modem ID. Hagenuk Speed Dragon?" );
+	    mis = mdm_get_idstring( "ATI0", 1, fd );
+	    break;
+	  case 1496:
+	    lprintf( L_MESG, "ZyXEL 1496 detected" ); 
+	    modem_type=Mt_class2_0;
+	    mis = mdm_get_idstring( "ATI1", 2, fd );
+	    break;
+	  case 2864:
+	    lprintf( L_MESG, "ZyXEL 2864(D) detected" );
+	    modem_type=Mt_class2_0;
+	    mis = mdm_get_idstring( "ATI1", 2, fd );
+	    break;
+	  case 28641:
+	  case 28642:
+	    lprintf( L_MESG, "ZyXEL 2864I(D) detected" );
+	    modem_type=Mt_class2_0;
+	    mis = mdm_get_idstring( "ATI1", 2, fd );
+	    break;
+	  case 1281:
+	  case 1291:
+	  case 1292:
+	  case 1293:
+	    lprintf( L_MESG, "ZyXEL Omni.NET detected" );
+	    modem_type=Mt_data;				/* has no fax mode */
+	    mis = mdm_get_idstring( "ATI1", 1, fd );
+	    break;
+	  case 1444:
+	  case 1445:
+	    lprintf( L_MESG, "USR Courier/Sportster v32bis detected (assuming non-fax capable)" );
+	    modem_type=Mt_data;
+	    break;
+	  case 2886:
+	  case 3361:
+	  case 3366:
+	    lprintf( L_MESG, "USR Courier/Sportster V.34(+) detected" );
+	    modem_type=Mt_class2_0;
+	    mis = mdm_get_idstring( "ATI3", 1, fd );
+	    break;
+	  case 5601:
+	  case 5607:
+	    lprintf( L_MESG, "USR Courier/Sportster 56k detected" );
+	    modem_type=Mt_class2_0;
+	    mis = mdm_get_idstring( "ATI3", 1, fd );
+	    break;
+	  case 6401:
+	    lprintf( L_MESG, "USR I-Modem detected" );
+	    modem_type=Mt_class2_0;
+	    mis = mdm_get_idstring( "ATI3", 1, fd );
+	    break;
+	  case 62:	/* sure? */
+	  case 962:
+	    lprintf( L_MESG, "Dr. Neuhaus Smarty detected (?)" );
+	    modem_type=Mt_class2;	/* now do ATI9! */
+	    modem_quirks |= MQ_NEED2;
+	    mis = mdm_get_idstring( "ATI9", 1, fd );
+	    break;
+	  case 961:
+	    lprintf( L_MESG, "Zoltrix 14400 faxmodem detected (?)" );
+	    modem_type=Mt_class2;
+	    mis = mdm_get_idstring( "ATI4", 1, fd );
+	    break;
+	  case 184:	/* sure? */
+	    lprintf( L_MESG, "Telebit FastBlazer detected" );
+	    modem_type=Mt_data;
+	    break;
+	  case 149: /* sure? */
+	    lprintf( L_MESG, "Intel 14.4E/400e detected (??)" );
+	    modem_type=Mt_unknown;
+	    break;
+	  case 247: /* use ATI2 for further distinction */
+	    lprintf( L_MESG, "Multitech MT1432BA/MT1932ZDX/MT2834ZDX detected" );
+	    modem_type=Mt_class2_0;
+	    mis = mdm_get_idstring( "ATI2", 1, fd );
+	    modem_quirks |= MQ_FBOR_OK;
+	    break;
+	  case 251: /* sure? */
+	    lprintf( L_MESG, "Discovery 2400 AM detected" );
+	    modem_type=Mt_data;
+	    break;
+	  case 641: /* sure? */
+	    lprintf( L_MESG, "ELSA MicroLink ISDN/TLpro detected" );
+	    modem_type=Mt_data;
+	    mis = mdm_get_idstring( "ATI3", 1, fd );
+	    break;
+	  case 282:	/* ATI6/ATI3 for model/firmware info */
+	    lprintf( L_MESG, "ELSA MicroLink 28.8 TQV detected" );
+	    modem_type=Mt_class2_0;
+	    mis = mdm_get_idstring( "ATI3", 1, fd );
+	    break;
+	  case 249:
+	  case 14400: /* further distinction necessary (ATI3,4,6)! */
+	  case 28800:
+	  case 33600:
+	    lprintf( L_MESG, "Generic Rockwell modem (%d)", mid );
+	    modem_type=Mt_class2;
+	    mis = mdm_get_idstring( "ATI3", 1, fd );
+	    if ( mid == 28800 && mis[0] == '\0' )	/* no ATI3 code */
+	    {
+		lprintf( L_MESG, "Sounds more like Dr.Neuhaus Cybermod" );
+		modem_quirks |= MQ_NEED2;
+	    }
+	    else					/* "Version 6.00" */
+	      if ( mid == 28800 && strncmp( mis, "Version", 7 ) == 0 )
+	    {
+		lprintf( L_MESG, "Could be a Hayes Optima/Accura modem" );
+	        mis = mdm_get_idstring( "ATI7", 2, fd );
+		modem_quirks |= MQ_NEED2;
+		break;
+	    }
+	    mis = mdm_get_idstring( "ATI4", 1, fd );
+	    break;
+	  default:
+	    lprintf( L_MESG, "unknown numerical modem id %d", mid );
+	    break;
+	}
+    }
+    else		/* non-numeric modem id string */
+    {
+	lprintf( L_MESG, "non-numeric ID string: '%s'", l );
+
+	/* "Elink 310 Version 1.25" */
+	if ( strncmp( l, "Elink", 5 ) == 0 )
+	{
+	    lprintf( L_MESG, "Elink detected" );
+	    modem_type=Mt_data;
+	}
+	else if ( strncmp( l, "Linux ISDN", 10 ) == 0 )
+	{
+	    lprintf( L_MESG, "ISDN4Linux detected" );
+	    modem_type=Mt_data;
+	}
+	/* got this from bruce@hn.pl.net */
+	else if ( strncmp( l, "1.03", 4 ) == 4 )
+	{
+	    lprintf( L_MESG, "Ellcon 14.4+Voice detected" );
+	    modem_type=Mt_data;
+	}
+    }
+
+    if ( mis != NULL ) 
+	lprintf( L_MESG, "additional info: '%s'", mis );
+
+    return NOERROR;
+}
