@@ -1,4 +1,4 @@
-#ident "@(#)faxsend.c	3.4 96/01/03 Copyright (c) 1994 Gert Doering"
+#ident "@(#)faxsend.c	4.3 97/03/20 Copyright (c) 1994 Gert Doering"
 
 /* faxsend.c
  *
@@ -30,7 +30,7 @@
 
 static boolean fax_sendpg_timeout = FALSE;
 
-static RETSIGTYPE fax_send_timeout()
+static RETSIGTYPE fax_send_timeout(SIG_HDLR_ARGS)
 {
     signal( SIGALRM, fax_send_timeout );	/* reactivate handler */
     
@@ -80,7 +80,8 @@ static void fax_send_panic_exit _P1( (fd), int fd )
     /* Hope that mgetty will be able to reinitialize it */
 
     alarm( 0 );
-    exit(11);
+    rmlocks();
+    exit(15);
 }
 
 /* fax_send_page - send one complete fax-G3-file to the modem
@@ -173,7 +174,7 @@ int fax_send_page _P5( (g3_file, bytes_sent, tio, ppm, fd),
 	int w_refresh = 0;
 	boolean first = TRUE;
 
-	alarm( 30 );		/* timeout if we get stuck in flow control */
+	alarm( 40 );		/* timeout if we get stuck in flow control */
 
 	while ( ( r = read( g3fd, buf, 64 ) ) > 0 )
 	{
@@ -325,16 +326,15 @@ int fax_send_ppm _P3( (fd, tio, ppm),
 	    ppm_char = 0x3b; break;
 	  case pp_eop:		/* no more pages or documents */
 	    /* stop being sensitive to DCD drops */
-#ifndef FAX_SEND_IGNORE_CARRIER
 #ifdef sun
-	    /* HW handshake has to be off while carrier is low */
-	    /* to avoid overruns, drain buffers first (Nils Jonsson) */
+	    /* On SUNs, HW handshake has to be off while carrier is low */
+	    /* -> to avoid underruns, drain buffers first (Nils Jonsson) */
 	    tio_drain_output( fd );
 	    tio_set_flow_control(fd, tio, (FAXSEND_FLOW) & FLOW_XON_OUT);
 #endif
 	    tio_carrier( tio, FALSE );
 	    tio_set( fd, tio );
-#endif
+
 	    ppm_char = 0x2e; break;
 	  default:
 	    lprintf( L_WARN, "ppm type %d not implemented", ppm );
@@ -404,14 +404,13 @@ int fax_send_ppm _P3( (fd, tio, ppm),
 	    /* take care of modems pulling DCD low before the final
 	     * result code has reached the host
 	     */
-#ifndef FAX_SEND_IGNORE_CARRIER
 	    tio_carrier( tio, FALSE );
 #ifdef sun
 	    /* HW handshake has to be off while carrier is low */
 	    tio_set_flow_control(fd, tio, (FAXSEND_FLOW) & FLOW_XON_OUT);
 #endif
 	    tio_set( fd, tio );
-#endif
+
 	    rc = fax_command( "AT+FET=2", "OK", fd );
 	    break;
 	  default:		/* pri-xxx codes */
