@@ -14,13 +14,19 @@ fi
 if expr "$VS" : '[0-9].[13579]' >/dev/null ; then
     DST=mgetty$VS-`date +%b%d`.tar.gz
 else
-    DST=$SRC
+    DST=mgetty+sendfax-$VS.tar.gz
 fi
 
+# find diff's, if any...
+DIFF=`ls -rt mgetty*-$VS.diff 2>/dev/null |tail -1`
+test -n "$DIFF" && DIFF="put $DIFF"
+
+# normal FTP upload
 ftp -v $HOST <<EOF
 cd $DIR
 bin
 hash
 put $SRC $DST
+$DIFF
 quit
 EOF
