@@ -1,4 +1,4 @@
-#ident "@(#)goodies.c	3.6 96/03/10 Copyright (c) 1993 Gert Doering"
+#ident "@(#)goodies.c	4.1 97/01/12 Copyright (c) 1993 Gert Doering"
 
 /*
  * goodies.c
@@ -205,9 +205,9 @@ char * get_ps_args _P1 ((pid), int pid )
 #endif
 }
 
-#if defined(NeXT) || defined(NEED_PUTENV)
+#if defined(NEED_STRDUP)
 
-/* provide strdup() for NEXTSTEP */
+/* provide strdup() for systems not having it... */
 char * strdup _P1( (src), char *src)
 {
 char * dest;
@@ -219,15 +219,21 @@ char * dest;
     return(dest);
 }
 
+#endif
+
+
+#if defined(NEED_PUTENV)
 
 /* provide putenv() for NEXTSTEP:
  * original code by Terrence W. Holm (tholm@uvicctr.UUCP),
  * slightly modified by Karl Berry (karl@cs.umb.edu)
- * contributed to mgetty by Georg Hoffleit (flight@mathi.uni-heidelberg.DE)
+ * contributed to mgetty by Gregor Hoffleit (flight@mathi.uni-heidelberg.DE)
  */
 
 #define  PSIZE  sizeof(char *)
 
+extern char **environ;
+
 int putenv _P1( (entry), char *entry)
 {
 unsigned length, size;
