@@ -1,4 +1,4 @@
-#ident "@(#)logfile.c	3.5 96/07/01 Copyright (c) Gert Doering"
+#ident "@(#)logfile.c	4.3 97/04/06 Copyright (c) Gert Doering"
 
 #include <stdio.h>
 #include <unistd.h>
@@ -18,7 +18,8 @@
 #ifdef SYSLOG
 #include <syslog.h>
 
-#if !defined(linux) && !defined(BSD) && !defined(SVR42) && !defined(solaris2)
+#if !defined(linux) && !defined(BSD) && !defined(_SCO_DS) && \
+    !defined(SVR42) && !defined(solaris2) && !defined(_AIX)
 int openlog _PROTO(( char *, int, int ));
 int syslog _PROTO(( int, char *, ... ));
 #endif
@@ -52,7 +53,7 @@ extern int atexit _PROTO(( void (*)(void) ));
    of those systems that _do_ declare them, it won't hurt */
 
 extern int sys_nerr;
-#if !defined(__NetBSD__) && !defined( __FreeBSD__ )
+#if !defined(__NetBSD__) && !defined( __FreeBSD__ ) && !defined(__GLIBC__)
 extern char *sys_errlist[];
 #endif
 
@@ -193,6 +194,7 @@ time_t  ti;
 struct tm *tm;
 va_list pvar;
 int     errnr;
+char * p;
 
     if ( level > log_level )	/* log level high enough? */
     {
@@ -271,6 +273,12 @@ int     errnr;
     
     va_end( pvar );
 
+    /* convert non-printable characters "in-place" to "_" */
+    for( p=ws; *p!='\0'; p++ )
+    {
+	if ( ! isprint(*p) ) *p='_';
+    }
+
     ti = time(NULL); tm = localtime(&ti);
 
     if ( level == L_AUDIT )		/* some little auditing */
