@@ -1,4 +1,4 @@
-#ident "@(#)mgetty.c	3.22 96/10/05 Copyright (c) Gert Doering"
+#ident "@(#)mgetty.c	4.8 97/05/24 Copyright (c) Gert Doering"
 
 /* mgetty.c
  *
@@ -87,11 +87,14 @@ chat_action_t	* answer_chat_actions = &ring_chat_actions[1];
 /* prototypes for system functions (that are missing in some 
  * system header files)
  */
+#ifndef __NetBSD__
 time_t		time _PROTO(( long * tloc ));
+#endif
 
 /* logname.c */
 int getlogname _PROTO(( char * prompt, TIO * termio,
-		char * buf, int maxsize, boolean do_timeout ));
+		char * buf, int maxsize, int max_login_time, 
+		boolean do_fido ));
 
 /* conf_mg.c */
 void exit_usage _PROTO((int num));
@@ -104,12 +107,14 @@ extern time_t	call_start;		/* time when we sent ATA */
 
 void gettermio _PROTO((char * tag, boolean first, TIO * tio));
 
+/* "simulated RING" handler */
 boolean virtual_ring = FALSE;
-static RETSIGTYPE sig_pick_phone()		/* "simulated RING" handler */
+static RETSIGTYPE sig_pick_phone(SIG_HDLR_ARGS)
 {
     signal( SIGUSR1, sig_pick_phone );
     virtual_ring = TRUE;
 }
+/* handle other signals: log them, and say goodbye... */
 static RETSIGTYPE sig_goodbye _P1 ( (signo), int signo )
 {
     lprintf( L_AUDIT, "failed dev=%s, pid=%d, got signal %d, exiting",
@@ -158,7 +163,7 @@ enum mgetty_States
    } mgetty_state = St_unknown;
 
 /* called on SIGUSR2. Exit, if no user online, ignore otherwise */
-static RETSIGTYPE sig_new_config()
+static RETSIGTYPE sig_new_config(SIG_HDLR_ARGS)
 {
     signal( SIGUSR2, sig_new_config );
     if ( mgetty_state != St_answer_phone &&
@@ -378,6 +383,7 @@ int main _P2((argc, argv), int argc, char ** argv)
     mgetty_get_config( Device );
 
 #ifdef VOICE
+    check_system();
     voice_config("vgetty", DevID);
     voice_register_event_handler(vgetty_handle_event);
 #endif
@@ -538,7 +544,8 @@ int main _P2((argc, argv), int argc, char ** argv)
 	     * /etc/nologin file), do some cleanups, and go back to
 	     * field one: St_waiting
 	     */
-	    CallTime = CallName = CallerId = "";	/* dirty */
+	    CallTime = CallName = "";			/* dirty */
+	    CallerId = "none";
 	    clean_line( STDIN, 3);			/* let line settle */
 	    rmlocks();
 	    mgetty_state = St_waiting;
@@ -561,7 +568,9 @@ int main _P2((argc, argv), int argc, char ** argv)
 	     */
 	    if ( ! c_bool(blocking) )
 	    {
-		if ( ! wait_for_input( STDIN, c_int(modem_check_time)*1000 ) &&
+		int wait_time = c_int(modem_check_time)*1000;
+
+		if ( ! wait_for_input( STDIN, wait_time ) &&
 		     ! c_bool(direct_line) && ! virtual_ring )
 		{
 		    /* no activity - is the modem alive or dead? */
@@ -723,20 +732,16 @@ int main _P2((argc, argv), int argc, char ** argv)
 			      &what_action,
 			      ( c_bool(ringback) && rings == 0 )
 				    ? c_int(ringback_time) : ring_chat_timeout,
-			      TRUE ) == FAIL 
+			      TRUE ) == FAIL
 #ifdef DIST_RING
-		    && (what_action != DIST_RING_VOICE)
+		    && ( what_action < A_RING1 || what_action > A_RING5 )
 #endif
 		   ) break;
 		rings++;
 	    }
 
 	    /* enough rings? */
-	    if ( rings >= rings_wanted
-#ifdef DIST_RING
-		|| ( what_action >= A_RING1 && what_action <= A_RING5 )
-#endif
-		)
+	    if ( rings >= rings_wanted )
 	    {
 		mgetty_state = St_answer_phone; break;
 	    }
@@ -825,7 +830,14 @@ Ring_got_action:
 		   called. If the function returns, the modem is ready
 		   to be connected in DATA mode with ATA. */
 		
-		vgetty_answer(rings, rings_wanted, what_action );
+		if ( vgetty_answer(rings, rings_wanted, what_action) ==
+		 VMA_CONNECT )
+		{
+		/* unexpected "CONNECT" -> jump to reading of login name
+		 */
+		    mg_get_ctty( STDIN, devname );
+		    mgetty_state = St_get_login; break;
+		};
 	    }
 #endif /* VOICE */
 
@@ -888,6 +900,14 @@ Ring_got_action:
 	    faxrec( FAX_SPOOL_IN, c_int(switchbd),
 		    uid, gid, c_int(fax_mode),
 		    c_isset(notify_mail)? c_string(notify_mail): NULL );
+
+    /* some modems require a manual hangup, with a pause before it. Notably
+       this is the creatix fax/voice modem, which is quite widespread,
+       unfortunately... */
+
+	    delay(1500);
+	    mdm_command( "ATH0", STDIN );
+
 	    rmlocks();
 	    exit( 0 );
 	    break;
@@ -936,6 +956,10 @@ Ring_got_action:
      */
     for (;;)
     {
+	/* protect against blocked output (for whatever reason) */
+	signal( SIGALRM, sig_goodbye );
+	alarm( 60 );
+
 	/* set ttystate for /etc/issue ("before" setting) */
 	gettermio(c_string(gettydefs_tag), TRUE, &tio);
 
@@ -954,6 +978,8 @@ Ring_got_action:
 	{
 	    /* display ISSUE, if desired
 	     */
+	    lprintf( L_NOISE, "print welcome banner (%s)", c_string(issue_file));
+
 	    if (c_string(issue_file)[0] == '!')		/* exec */
             {
                 system( c_string(issue_file)+1 );
@@ -988,6 +1014,9 @@ Ring_got_action:
 		tio.c_iflag, tio.c_oflag, tio.c_cflag, tio.c_lflag,
 		c_string(login_prompt));
 #endif
+	/* turn off alarm (getlogname has its own timeout) */
+	alarm(0);
+
 	/* read a login name from tty
 	   (if just <cr> is pressed, re-print issue file)
 
@@ -996,7 +1025,11 @@ Ring_got_action:
 	   for c_oflag */
 
 	if ( getlogname( c_string(login_prompt), &tio, buf, sizeof(buf), 
-			 !c_bool(blocking) ) == -1 ) continue;
+			 c_bool(blocking)? 0: c_int(login_time), 
+			 c_bool(do_send_emsi) ) == -1 ) 
+	{
+	     continue;
+	}
 
 	/* remove PID file (mgetty is due to exec() login) */
 #ifdef MGETTY_PID_FILE
