@@ -1,7 +1,7 @@
 #ifndef ___MGETTY_H
 #define ___MGETTY_H
 
-#ident "@(#)mgetty.h	3.12 96/10/10 Copyright (c) Gert Doering"
+#ident "@(#)mgetty.h	4.4 97/06/28 Copyright (c) Gert Doering"
 
 /* mgetty.h
  *
@@ -82,13 +82,27 @@ int lprintf _PROTO(());
  * since the tv_usec timer is not exact at all on SCO.
  * If your System has the "nap(S)" call, you can use this instead of
  * select(S) or poll(S) for sleeping less than one second.
+ * Ditto for usleep(S), for systems having it.
  */
+#if defined(linux) || defined(_AIX)
+# define USE_USLEEP
+#endif
 
 #if !defined(USE_POLL) && !defined(USE_READ)
 #define USE_SELECT
 #endif
 
+/* these definitions specify the return value type and the arguments
+ * for signal handler functions - if your compiler barfs, change them
+ */
 typedef	void	RETSIGTYPE;
+
+#ifdef _SCO_DS		/* SCO OSR 5.0 */
+# define SIG_HDLR_ARGS	int signo
+#else
+# define SIG_HDLR_ARGS
+#endif
+
 typedef	char	boolean;
 
 /* the cpp directive "sun" isn't useful at all, since is defined on
@@ -138,9 +152,10 @@ typedef	char	boolean;
 #endif
 
 /* On a ALPHA, the config routines won't work, unless we change the
- * union to use "void *" and "long", instead of "int" (see config.h). 
+ * union to use "void *" and "long", instead of "int" (see config.h).
+ * Same for Sparc Ultra machines [at least with SparcLinux]
  */
-#if defined(__alpha__)
+#if defined(__alpha__) || defined(__sparc64__)
 # define PTR_IS_LONG
 #endif
 
@@ -189,6 +204,7 @@ int		steal_lock _PROTO((char * device, int pid ));
 void	faxrec _PROTO(( char * spool_dir, unsigned int switchbd,
 		        int uid, int gid, int mode, char * mail_to ));
 char *	fax_strerror _PROTO(( int fax_hangup_code ));
+void	faxlib_init _PROTO((void));
 
 /* initialization stuff: mg_m_init.c */
 int	mg_init_data  _PROTO(( int fd, char * chat_seq[], char * force_seq[]));
@@ -211,6 +227,7 @@ int	mdm_send _PROTO(( char * send, int fd ));
 int	mdm_read_byte _PROTO(( int fd, char * c ));
 char *	mdm_get_line  _PROTO(( int fd ));
 int	mdm_command   _PROTO(( char * send, int fd ));
+char *  mdm_get_idstring _PROTO(( char * send, int n, int fd ));
 
 /* logname.c */
 char *	ln_escape_prompt _PROTO(( char * prompt ));
@@ -275,4 +292,11 @@ extern char *	optarg;
     typedef ushort gid_t;
 #endif
 
+#if defined(NeXT)
+# define NEED_PUTENV
+# define NEED_STRDUP
+
+ char * strdup _PROTO(( char *src ));
+#endif
+
 #endif			/* ___MGETTY_H */
