@@ -1,10 +1,14 @@
-#ident "@(#)mksed.c	3.3 96/04/11 Copyright (c) Gert Doering"
+#ident "@(#)mksed.c	4.2 97/02/17 Copyright (c) Gert Doering"
 
 #include <stdio.h>
 
 #include "mgetty.h"
 #include "policy.h"
 
+#ifndef MAIL_TO
+# define MAIL_TO "root"
+#endif
+
 int main _P0( void )
 {
     printf( "sed \\\n" );
@@ -15,9 +19,13 @@ int main _P0( void )
     printf( "      -e 's;@FAX_MODEM_TTYS@;%s;g'\\\n", FAX_MODEM_TTYS );
     printf( "      -e 's;@FAX_STATION_ID@;%s;g'\\\n", FAX_STATION_ID );
     printf( "      -e 's;@FAX_LOG@;%s;g'\\\n", FAX_LOG );
+    printf( "      -e 's;@LOG_LEVEL@;%d;g'\\\n", LOG_LEVEL );
     printf( "      -e 's;@LOG_PATH@;");
         printf( LOG_PATH, "ttyxx" );
         printf( ";g'\\\n" );
+    printf( "      -e 's;@NOLOGIN@;" );
+	printf( NOLOGIN_FILE, "ttyxx" );
+        printf( ";g'\\\n" );
     printf( "      -e 's;@KVG_PID_FILE@;");
 #ifdef MGETTY_PID_FILE
         printf( MGETTY_PID_FILE, "$TTY" );
