@@ -1,4 +1,4 @@
-#ident "@(#)sendfax.c	3.12 96/09/15 Copyright (c) Gert Doering"
+#ident "@(#)sendfax.c	4.5 97/07/13 Copyright (c) Gert Doering"
 
 /* sendfax.c
  *
@@ -40,7 +40,9 @@ boolean	verbose = FALSE;
 extern time_t	call_start;			/* for accounting */
 
 /* seems to missing nearly everywhere */
+#ifndef __NetBSD__
 time_t	time _PROTO(( long * tloc ));
+#endif
 
 void exit_usage _P2( (program, msg ),
 		     char * program, char * msg )
@@ -204,7 +206,8 @@ void fax_close _P1( (fd),
 		    int fd )
 {
     fax_send( "AT+FCLASS=0", fd );
-    delay(100);
+    delay(500);
+    tio_flush_queue( fd, TIO_Q_BOTH );		/* unlock flow ctl. */
     close( fd );
     rmlocks();
 }
@@ -353,12 +356,27 @@ int main _P2( (argc, argv),
     siginterrupt( SIGHUP,  TRUE );
 #endif
 
+    /* now set speed for this port (do this *after* sendfax_get_config())!
+     */
+    if ( tio_set_speed( &fax_tio, c_int(speed) ) == ERROR ||
+         tio_set( fd, &fax_tio ) == ERROR )
+    {
+	fprintf( stderr, "%s: cannot set serial port speed %d on \"%s\"\n",
+			argv[0], c_int(speed), Device );
+	close(fd);
+	rmlocks();
+	exit(2);
+    }
+
+
     if ( fax_command( "ATV1Q0", "OK", fd ) == ERROR )
     {
 	lprintf( L_ERROR, "modem doesn't talk to me" );
 	fprintf( stderr, "The modem doesn't respond!\n" );
-	close( fd );
-	exit( 3 );
+	tio_flush_queue( fd, TIO_Q_BOTH );	/* unlock flow ctl. */
+	close(fd);
+	rmlocks();
+	exit(3);
     }
 
     /* extra initialization: -m / modem-init */
@@ -391,6 +409,14 @@ int main _P2( (argc, argv),
 	exit( 3 );
     }
 
+    if ( modem_type == Mt_data )
+    {
+	lprintf( L_ERROR, "modem can't do class 2 or class 2.0" );
+	fprintf( stderr, "%s: not a class 2/2.0 fax modem\n", argv[0] );
+	fax_close( fd );
+	exit( 3 );
+    }
+
     /* some modems need a baud rate switch after +FCLASS=2,
      * see policy.h for details
      */
@@ -490,20 +516,23 @@ int main _P2( (argc, argv),
     }
     if ( verbose ) printf( "OK.\n" );
 
-#ifndef FAX_SEND_IGNORE_CARRIER
-    /* by now, the modem should have raised DCD, so remove CLOCAL flag */
-    tio_carrier( &fax_tio, TRUE );
+    if ( c_bool( ignore_carrier ))	/* ignore carrier */
+    {
+	lprintf( L_MESG, "sendfax: IGNORE DCD (carrier) status" );
+    }
+    else				/* honour carrier */
+    {
+	/* by now, the modem should have raised DCD, so remove CLOCAL flag */
+	tio_carrier( &fax_tio, TRUE );
 
 #ifdef sun
-    /* now we can request hardware flow control since we have carrier */
-    tio_set_flow_control( fd, &fax_tio, (FAXSEND_FLOW) & (FLOW_HARD|FLOW_XON_OUT) );
+	/* now we can request hardware flow control since we have carrier */
+	tio_set_flow_control( fd, &fax_tio, (FAXSEND_FLOW) & (FLOW_HARD|FLOW_XON_OUT) );
 #endif	/* sun */
-    tio_set( fd, &fax_tio );
+	tio_set( fd, &fax_tio );
 
-    lprintf( L_NOISE, "sendfax: honouring DCD (carrier) drops now" );
-#else
-    lprintf( L_NOISE, "sendfax: FAX_SEND_IGNORE_CARRIER active" );
-#endif	/* !FAX_SEND_IGNORE_CARRIER */
+	lprintf( L_MESG, "sendfax: honouring DCD (carrier) drops now" );
+    }
 
     total_pages = argc-argidx;		/* for statistics */
 
