@@ -1,4 +1,4 @@
-#ident "@(#)tio.h	3.10 96/10/10 Copyright (c) 1993 Gert Doering"
+#ident "@(#)tio.h	4.1 97/01/12 Copyright (c) 1993 Gert Doering"
 
 #ifndef __TIO_H__
 #define __TIO_H__
@@ -51,6 +51,13 @@ typedef struct sgttyb TIO;
 # include <sys/fas.h>
 #endif
 
+/* make sure <sys/ioctl.h> gets included: contains TIOCM* definitions
+ * on AIX, and ioctl() prototype on NeXT and Linux
+ */
+#if defined(_AIX) || defined(NeXT) || defined(linux)
+# include <sys/ioctl.h>
+#endif
+
 /* define some types for gettydefs.c */
 
 #ifdef SYSV_TERMIO
