@@ -1,4 +1,4 @@
-#ident "@(#)utmp.c	3.2 96/06/25 Copyright (c) Gert Doering"
+#ident "@(#)utmp.c	4.3 97/05/27 Copyright (c) Gert Doering"
 
 /* some parts of the code (writing of the utmp entry)
  * is based on the "getty kit 2.0" by Paul Sutcliffe, Jr.,
@@ -138,7 +138,7 @@ FILE *	fp;
 	    }
 #endif		/* SVR4 */
 
-#ifdef M_UNIX
+#if defined(M_UNIX) || defined(__GLIBC__)
 	    if ( pututline(utmp) == NULL )
 	    {
 		lprintf( L_ERROR, "cannot create utmp entry" );
@@ -154,6 +154,9 @@ FILE *	fp;
 #ifdef SVR4
 	    updwtmpx(WTMPX_FILE, utmp);
 #else
+# if defined(__GLIBC__) && __GLIBC__ >= 2
+	    updwtmp(WTMP_FILE, utmp);
+# else
 	    if (stat(WTMP_FILE, &st) && errno == ENOENT)
 		    break;
 	    if ((fp = fopen(WTMP_FILE, "a")) != (FILE *) NULL)
@@ -162,6 +165,7 @@ FILE *	fp;
 		(void) fwrite((char *)utmp,sizeof(*utmp),1,fp);
 		(void) fclose(fp);
 	    }
+# endif	/* GNU Libc 2.x */
 #endif	/* !SVR4 */
 
 	    lprintf(L_NOISE, "utmp + wtmp entry made");
