@@ -7,20 +7,20 @@
 
 #include "../include/voice.h"
 
-char *libvoice_mode_c = "$Id: mode.c,v 1.3 1996/07/25 19:23:25 marc Exp $";
+char *libvoice_mode_c = "$Id: mode.c,v 1.5 1996/10/30 06:19:03 marc Exp $";
 
 int voice_mode_on _P0(void)
      {
      lprintf(L_NOISE, "%s: entering voice mode", program_name);
      voice_install_signal_handler();
-     voice_modem_handle_event(VOICE_MODE_ON, (event_data) 0);
+     voice_modem->voice_mode_on();
      return(OK);
-     };
+     }
 
 int voice_mode_off _P0(void)
      {
      lprintf(L_NOISE, "%s: leaving voice mode", program_name);
-     voice_modem_handle_event(VOICE_MODE_OFF, (event_data) 0);
+     voice_modem->voice_mode_off();
      voice_restore_signal_handler();
      return(OK);
-     };
+     }
