@@ -5,7 +5,7 @@
 
 #include "../include/voice.h"
 
-char *vgetty_answer_c = "$Id: answer.c,v 1.6 1996/07/25 19:24:31 marc Exp $";
+char *vgetty_answer_c = "$Id: answer.c,v 1.17 1997/04/11 15:07:34 marc Exp $";
 
 int answer_mode;
 char dtmf_code[VOICE_BUF_LEN];
@@ -14,7 +14,7 @@ int first_dtmf;
 int hangup_requested;
 int switch_to_data_fax_mode;
 
-static void enter_data_fax_mode _P1((answer_mode), int answer_mode)
+static int enter_data_fax_mode _P1((answer_mode), int answer_mode)
      {
      int bit_order = 0;
      char *fax_mode = NULL;
@@ -23,10 +23,10 @@ static void enter_data_fax_mode _P1((answer_mode), int answer_mode)
 
      if (answer_mode == 0)
           {
-          lprintf(L_MESG,
+          lprintf(L_NOISE,
            "%s: answer mode is set to not accept data or fax connections",
            program_name);
-          exit(1);
+          exit(99);
           };
 
      if (cvd.force_autodetect.d.i)
@@ -49,9 +49,9 @@ static void enter_data_fax_mode _P1((answer_mode), int answer_mode)
 
      if (answer_mode == 0)
           {
-          lprintf(L_MESG, "%s: modem does not support wanted answer mode",
+          lprintf(L_NOISE, "%s: modem does not support wanted answer mode",
            program_name);
-          exit(1);
+          return(FAIL);
           };
 
      switch (answer_mode)
@@ -59,34 +59,60 @@ static void enter_data_fax_mode _P1((answer_mode), int answer_mode)
           case ANSWER_DATA | ANSWER_FAX:
                lprintf(L_JUNK, "%s: trying data and fax connection",
                 program_name);
-               voice_switch_to_data_fax((void *) fax_mode);
-               voice_command("AT+FAA=1", "OK");
+
+               if (voice_switch_to_data_fax(fax_mode) == FAIL)
+                    return(FAIL);
+
+               if (voice_command("AT+FAA=1", "OK") != VMA_USER_1)
+                    return(FAIL);
+
                fax_set_bor(voice_fd, bit_order);
                break;
           case ANSWER_DATA:
                lprintf(L_JUNK, "%s: trying data connection", program_name);
-               voice_switch_to_data_fax((void *) "0");
-               voice_command("AT+FAA=0", "OK");
+
+               if (voice_switch_to_data_fax("0") == FAIL)
+                    return(FAIL);
+
+               if (voice_command("AT+FAA=0", "OK") != VMA_USER_1)
+                    return(FAIL);
+
                break;
           case ANSWER_FAX:
                lprintf(L_JUNK, "%s: trying fax connection", program_name);
-               voice_switch_to_data_fax((void *) fax_mode);
-               voice_command("AT+FAA=0", "OK");
+
+               if (voice_switch_to_data_fax(fax_mode) == FAIL)
+                    return(FAIL);
+
+               if (voice_command("AT+FAA=0", "OK") != VMA_USER_1)
+                    return(FAIL);
+
                fax_set_bor(voice_fd, bit_order);
                break;
           };
 
-     };
+     voice_restore_signal_handler();
+     return(OK);
+     }
 
-static int get_answer_mode _P0(void)
+static int get_answer_mode _P1((ring_type), char* ring_type)
      {
      char answer_mode[VOICE_BUF_LEN] = "";
      int result = 0;
 
-     if (virtual_ring)
-          strcpy(answer_mode, cvd.sigusr1_answer_mode.d.p);
-     else
-          strcpy(answer_mode, cvd.ring_answer_mode.d.p);
+     /*
+      * This is only a temporary place for the read from the config file,
+      * to keep the interface to mgetty unchanged. It will move to another
+      * routine in the future, that is called from mgetty after the first
+      * ring, when the ring type and caller id information is available,
+      * so that the number of rings to answer the phone can also be set by
+      * the voice configuration file.
+      */
+
+     lprintf(L_MESG, "reading ring_type %s configuration from config file %s",
+      ring_type, voice_config_file);
+     get_config(voice_config_file, (conf_data *) &cvd, "ring_type", ring_type);
+     strcpy(answer_mode, cvd.answer_mode.d.p);
 
      if (strlen(answer_mode) != 0)
           {
@@ -161,15 +187,20 @@ static int get_answer_mode _P0(void)
 
      if (result == 0)
           {
-          lprintf(L_MESG,
+          lprintf(L_NOISE,
            "%s: answer mode is set to not accept any kind of call",
            program_name);
-          clean_line(STDIN, 80);
+
+          /*
+           * Ignore everything until the phone stops ringing and exit then
+           */
+
+          voice_flush(100);
           exit(1);
           };
 
      return(result);
-     };
+     }
 
 static void get_greeting_message _P1((message), char *message)
      {
@@ -194,25 +225,32 @@ static void get_greeting_message _P1((message), char *message)
           while (fgets(message_file_name, VOICE_BUF_LEN, list) != NULL)
                count++;
 
-          choice = time(NULL) % count + 1;
-          lprintf(L_JUNK, "%s: found %d messages, picked message number %d",
-           program_name, count, choice);
-          rewind(list);
+          if (count == 0)
+               strcpy(message_file_name, cvd.backup_message.d.p);
+          else
+               {
+               choice = time(NULL) % count + 1;
+               lprintf(L_JUNK,
+                "%s: found %d messages, picked message number %d",
+                program_name, count, choice);
+               rewind(list);
 
-          for (i = 0; i < choice; i++)
-               fgets(message_file_name, VOICE_BUF_LEN, list);
+               for (i = 0; i < choice; i++)
+                    fgets(message_file_name, VOICE_BUF_LEN, list);
 
-          i = strlen(message_file_name) - 1;
+               i = strlen(message_file_name) - 1;
 
-          if ((i > 0) && (message_file_name[i] == NL))
-               message_file_name[i] = 0x00;
+               if ((i > 0) && (message_file_name[i] == NL))
+                    message_file_name[i] = 0x00;
+
+               }
 
           fclose(list);
           };
 
      make_path(message, message_dir, message_file_name);
      lprintf(L_JUNK, "%s: message name is %s", program_name, message);
-     };
+     }
 
 static void remove_message _P1((message), char *message)
      {
@@ -223,9 +261,9 @@ static void remove_message _P1((message), char *message)
           unlink(message);
           };
 
-     };
+     }
 
-void vgetty_answer _P3((rings, rings_wanted, ring_action), int rings,
+int vgetty_answer _P3((rings, rings_wanted, ring_action), int rings,
  int rings_wanted, action_t ring_action)
      {
      time_t call_start;
@@ -235,31 +273,50 @@ void vgetty_answer _P3((rings, rings_wanted, ring_action), int rings,
      char receive_dir[VOICE_BUF_LEN];
      char message[VOICE_BUF_LEN];
      char message_name[VOICE_BUF_LEN] = "vXXXXXX";
+     char *ring_type = "ring";
+     int result;
 
-     answer_mode = get_answer_mode();
+     /*
+      * Mapping from mgetty's answer mode detection to the new vgetty way
+      */
 
-     if ((ring_action >= A_RING1) && (ring_action <= A_RING5))
+     if (virtual_ring)
+          ring_type = "virtual";
 
-          if ((ring_action != (cvd.dist_ring_fax.d.i + A_RING1 - 1)) &&
-           (ring_action != (cvd.dist_ring_data.d.i + A_RING1 - 1)) &&
-           (ring_action != (cvd.dist_ring_voice.d.i + A_RING1 - 1)))
-               {
-               lprintf(L_AUDIT, "ignoring distinctive RING %d call",
-                ring_action - A_RING1 + 1);
-               clean_line(STDIN, 80);
-               exit(1);
-               };
-
-     if (ring_action == (cvd.dist_ring_fax.d.i + A_RING1 - 1))
+     switch (ring_action)
           {
-          enter_data_fax_mode(answer_mode & ANSWER_FAX);
-          return;
+          case A_RING1:
+               ring_type = "ring1";
+               break;
+          case A_RING2:
+               ring_type = "ring2";
+               break;
+          case A_RING3:
+               ring_type = "ring3";
+               break;
+          case A_RING4:
+               ring_type = "ring4";
+               break;
+          case A_RING5:
+               ring_type = "ring5";
+               break;
+          default:
+               break;
           };
 
-     if ((rings < rings_wanted) || !(answer_mode & ANSWER_VOICE))
+     answer_mode = get_answer_mode(ring_type);
+
+     if ((answer_mode & ANSWER_VOICE) == 0)
           {
-          enter_data_fax_mode(answer_mode);
-          return;
+
+          if (enter_data_fax_mode(answer_mode) == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not switch to data or fax mode",
+                program_name);
+               exit(99);
+               }
+
+          return(VMA_OK);
           };
 
      get_greeting_message(greeting_message);
@@ -273,8 +330,25 @@ void vgetty_answer _P3((rings, rings_wanted, ring_action), int rings,
      hangup_requested = FALSE;
      switch_to_data_fax_mode = FALSE;
 
-     voice_mode_on();
-     voice_answer_phone();
+     if (voice_mode_on() == FAIL)
+          {
+          lprintf(L_ERROR, "%s: Could not switch to voice mode",
+           program_name);
+          exit(99);
+          }
+
+     if ((result = voice_answer_phone()) == VMA_CONNECT)
+          {
+          voice_restore_signal_handler();
+          return(VMA_CONNECT);
+          }
+
+     if (result == VMA_ERROR)
+          {
+          lprintf(L_ERROR, "%s: Could not answer the phone. Strange...",
+           program_name);
+          exit(99);
+          }
 
      if (strlen(cvd.call_program.d.p) != 0)
           {
@@ -294,41 +368,111 @@ void vgetty_answer _P3((rings, rings_wanted, ring_action), int rings,
                switch (result)
                     {
                     case 1:
-                         enter_data_fax_mode(answer_mode & ANSWER_DATA);
-                         return;
+
+                         if (enter_data_fax_mode(answer_mode & ANSWER_DATA) ==
+                          FAIL)
+                              {
+                              lprintf(L_ERROR,
+                               "%s: Could not switch to data mode",
+                               program_name);
+                              exit(99);
+                              }
+
+                         return(VMA_OK);
                     case 2:
-                         enter_data_fax_mode(answer_mode & ANSWER_FAX);
-                         return;
+
+                         if (enter_data_fax_mode(answer_mode & ANSWER_FAX) ==
+                          FAIL)
+                              {
+                              lprintf(L_ERROR,
+                               "%s: Could not switch to fax mode",
+                               program_name);
+                              exit(99);
+                              }
+
+                         return(VMA_OK);
                     case 3:
-                         enter_data_fax_mode(answer_mode & (ANSWER_DATA |
-                          ANSWER_FAX));
-                         return;
+
+                         if (enter_data_fax_mode(answer_mode & (ANSWER_DATA |
+                          ANSWER_FAX)) == FAIL)
+                              {
+                              lprintf(L_ERROR,
+                               "%s: Could not switch to data/fax mode",
+                               program_name);
+                              exit(99);
+                              }
+
+                         return(VMA_OK);
                     default:
-                         voice_set_device(NO_DEVICE);
-                         voice_mode_off();
-                         voice_close_device();
+
+                         if (voice_set_device(NO_DEVICE) == FAIL)
+                              {
+                              lprintf(L_ERROR,
+                               "%s: Could not hang up the line",
+                               program_name);
+                              exit(99);
+                              }
+
+                         if (voice_mode_off() == FAIL)
+                              {
+                              lprintf(L_ERROR,
+                               "%s: Could not turn off voice mode",
+                               program_name);
+                              exit(99);
+                              }
+
+                         if (voice_close_device() == FAIL)
+                              {
+                              lprintf(L_ERROR,
+                               "%s: Could not close the voice device",
+                               program_name);
+                              exit(99);
+                              }
+
                          exit(0);
                     };
 
           };
 
-     voice_play_file(greeting_message);
+     if (voice_play_file(greeting_message) == FAIL)
+          lprintf(L_ERROR, "%s: Could not play greeting message",
+           program_name);
 
      if ((execute_dtmf_script) && (strlen(cvd.dtmf_program.d.p) != 0))
           {
-          char *arguments[2];
+          char *arguments[3];
           char dtmf_program[VOICE_BUF_LEN];
 
           arguments[0] = dtmf_code;
-          arguments[1] = NULL;
+          arguments[1] = "none";
+          arguments[2] = NULL;
           make_path(dtmf_program, cvd.voice_dir.d.p, cvd.dtmf_program.d.p);
           lprintf(L_AUDIT,
            "got DTMF code - executing DTMF program, name='%s', caller=%s, dev=%s, pid=%d",
            CallName, CallerId, DevID, getpid());
           voice_execute_shell_script(dtmf_program, arguments);
-          voice_set_device(NO_DEVICE);
-          voice_mode_off();
-          voice_close_device();
+
+          if (voice_set_device(NO_DEVICE) == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not hang up the line",
+                program_name);
+               exit(99);
+               }
+
+          if (voice_mode_off() == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not turn off voice mode",
+                program_name);
+               exit(99);
+               }
+
+          if (voice_close_device() == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not close the voice device",
+                program_name);
+               exit(99);
+               }
+
           exit(0);
           };
 
@@ -337,35 +481,82 @@ void vgetty_answer _P3((rings, rings_wanted, ring_action), int rings,
           lprintf(L_AUDIT,
            "hangup requested, name='%s', caller=%s, dev=%s, pid=%d",
            CallName, CallerId, DevID, getpid());
-          voice_set_device(NO_DEVICE);
-          voice_mode_off();
-          voice_close_device();
+
+          if (voice_set_device(NO_DEVICE) == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not hang up the line",
+                program_name);
+               exit(99);
+               }
+
+          if (voice_mode_off() == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not turn off voice mode",
+                program_name);
+               exit(99);
+               }
+
+          if (voice_close_device() == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not close the voice device",
+                program_name);
+               exit(99);
+               }
+
           exit(0);
           };
 
      if (switch_to_data_fax_mode)
           {
-          enter_data_fax_mode(answer_mode);
-          return;
+
+          if (enter_data_fax_mode(answer_mode) == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not switch to data/fax mode",
+                program_name);
+               exit(99);
+               }
+
+          return(VMA_OK);
           };
 
-     voice_beep(cvd.beep_frequency.d.i, cvd.beep_length.d.i);
+     if (voice_beep(cvd.beep_frequency.d.i, cvd.beep_length.d.i) == FAIL)
+          lprintf(L_ERROR, "%s: Beep command failed");
 
      if ((execute_dtmf_script) && (strlen(cvd.dtmf_program.d.p) != 0))
           {
-          char *arguments[2];
+          char *arguments[3];
           char dtmf_program[VOICE_BUF_LEN];
 
           arguments[0] = dtmf_code;
-          arguments[1] = NULL;
+          arguments[1] = "none";
+          arguments[2] = NULL;
           make_path(dtmf_program, cvd.voice_dir.d.p, cvd.dtmf_program.d.p);
           lprintf(L_AUDIT,
            "got DTMF code - executing DTMF program, name='%s', caller=%s, dev=%s, pid=%d",
            CallName, CallerId, DevID, getpid());
           voice_execute_shell_script(dtmf_program, arguments);
-          voice_set_device(NO_DEVICE);
-          voice_mode_off();
-          voice_close_device();
+
+          if (voice_set_device(NO_DEVICE) == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not hang up the line",
+                program_name);
+               exit(99);
+               }
+
+          if (voice_mode_off() == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not turn off voice mode",
+                program_name);
+               exit(99);
+               }
+
+          if (voice_close_device() == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not close the voice device",
+                program_name);
+               exit(99);
+               }
+
           exit(0);
           };
 
@@ -374,51 +565,143 @@ void vgetty_answer _P3((rings, rings_wanted, ring_action), int rings,
           lprintf(L_AUDIT,
            "hangup requested, name='%s', caller=%s, dev=%s, pid=%d",
            CallName, CallerId, DevID, getpid());
-          voice_set_device(NO_DEVICE);
-          voice_mode_off();
-          voice_close_device();
+
+          if (voice_set_device(NO_DEVICE) == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not hang up the line",
+                program_name);
+               exit(99);
+               }
+
+          if (voice_mode_off() == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not turn off voice mode",
+                program_name);
+               exit(99);
+               }
+
+          if (voice_close_device() == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not close the voice device",
+                program_name);
+               exit(99);
+               }
+
           exit(0);
           };
 
      if (switch_to_data_fax_mode)
           {
-          enter_data_fax_mode(answer_mode);
-          return;
+
+          if (enter_data_fax_mode(answer_mode) == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not switch to data/fax mode",
+                program_name);
+               exit(99);
+               }
+
+          return(VMA_OK);
           };
 
      time(&call_start);
-     voice_record_file(message);
+
+     if (voice_record_file(message) == FAIL)
+          {
+          lprintf(L_ERROR, "%s: Could not record a message",
+           program_name);
+          exit(99);
+          }
+
      time(&call_end);
      call_length = call_end - call_start;
 
+     if ((cvd.rec_min_len.d.i != 0) && (call_length <= cvd.rec_min_len.d.i))
+          switch_to_data_fax_mode = TRUE;
+
      if ((execute_dtmf_script) && (strlen(cvd.dtmf_program.d.p) != 0))
           {
-          char *arguments[2];
+          char *arguments[3];
           char dtmf_program[VOICE_BUF_LEN];
 
           arguments[0] = dtmf_code;
-          arguments[1] = NULL;
+          arguments[1] = message;
+          arguments[2] = NULL;
           remove_message(message);
           make_path(dtmf_program, cvd.voice_dir.d.p, cvd.dtmf_program.d.p);
           lprintf(L_AUDIT,
            "got DTMF code - executing DTMF program, name='%s', caller=%s, dev=%s, pid=%d",
            CallName, CallerId, DevID, getpid());
           voice_execute_shell_script(dtmf_program, arguments);
-          voice_set_device(NO_DEVICE);
-          voice_mode_off();
-          voice_close_device();
+
+          if (voice_set_device(NO_DEVICE) == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not hang up the line",
+                program_name);
+               exit(99);
+               }
+
+          if (voice_mode_off() == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not turn off voice mode",
+                program_name);
+               exit(99);
+               }
+
+          if (voice_close_device() == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not close the voice device",
+                program_name);
+               exit(99);
+               }
+
           exit(0);
           };
 
      if (!hangup_requested && (switch_to_data_fax_mode))
           {
           remove_message(message);
-          enter_data_fax_mode(answer_mode);
-          return;
+
+          if (enter_data_fax_mode(answer_mode) == FAIL)
+               {
+               lprintf(L_ERROR, "%s: Could not switch to data/fax mode",
+                program_name);
+               exit(99);
+               }
+
+          return(VMA_OK);
           };
 
      if (!hangup_requested)
-          voice_beep(cvd.beep_frequency.d.i, cvd.beep_length.d.i);
+
+          if (voice_beep(cvd.beep_frequency.d.i, cvd.beep_length.d.i) == FAIL)
+               lprintf(L_ERROR, "%s: Beep command failed");
+
+     if (voice_set_device(NO_DEVICE) == FAIL)
+          {
+          lprintf(L_ERROR, "%s: Could not hang up the line",
+           program_name);
+          exit(99);
+          }
+
+     if (voice_mode_off() == FAIL)
+          {
+          lprintf(L_ERROR, "%s: Could not turn off voice mode",
+           program_name);
+          exit(99);
+          }
+
+     if (voice_close_device() == FAIL)
+          {
+          lprintf(L_ERROR, "%s: Could not close the voice device",
+           program_name);
+          exit(99);
+          }
+
+     lprintf(L_AUDIT,
+      "message keep, length=%02d:%02d:%02d, name='%s', caller=%s, dev=%s, pid=%d",
+      (int) (call_length / 3600), (int) ((call_length / 60) % 60),
+      (int) (call_length % 60), CallName, CallerId, DevID, getpid());
+     vgetty_create_message_flag_file();
 
      if (strlen(cvd.message_program.d.p) != 0)
           {
@@ -429,20 +712,12 @@ void vgetty_answer _P3((rings, rings_wanted, ring_action), int rings,
           arguments[1] = CallerId;
           arguments[2] = CallName;
           arguments[3] = NULL;
-          lprintf(L_AUDIT, "executing message program, dev=%s, pid=%d",
+          lprintf(L_NOISE, "executing message program, dev=%s, pid=%d",
            DevID, getpid());
           make_path(message_program, cvd.voice_dir.d.p,
            cvd.message_program.d.p);
           voice_execute_shell_script(message_program, arguments);
           };
 
-     voice_set_device(NO_DEVICE);
-     voice_mode_off();
-     voice_close_device();
-     lprintf(L_AUDIT,
-      "message keep, length=%02d:%02d:%02d, name='%s', caller=%s, dev=%s, pid=%d",
-      (int) (call_length / 3600), (int) ((call_length / 60) % 60),
-      (int) (call_length % 60), CallName, CallerId, DevID, getpid());
-     vgetty_create_message_flag_file();
      exit(0);
-     };
+     }
